@@ -103,8 +103,10 @@
 #define LD_GUARD     (1u<<23)
 /** Generation and application of consensus diffs. */
 #define LD_CONSDIFF  (1u<<24)
+/** Denial of Service mitigation. */
+#define LD_DOS       (1u<<25)
 /** Number of logging domains in the code. */
-#define N_LOGGING_DOMAINS 25
+#define N_LOGGING_DOMAINS 26
 
 /** This log message is not safe to send to a callback-based logger
  * immediately.  Used as a flag, not a log domain. */
